unit uMain;
{$R *.dfm}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, dwgThumbnail;

type
  TfmViewerMain = class(TForm)
    ScrollBox1: TScrollBox;
    imgMain: TImage;
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    function OpenImage(FileName: string): Boolean;
  public
    constructor CreateParented(ParentWnd: HWND; FileName: string; var HandleResult: THandle; FitImgToWindow: Boolean);
    function GlobalLoadNextImage(FileName: string): Boolean;
  end;

type
  TPlugInfo = packed record
    PlugWinProc: Pointer; //callback function of our form
    PlugForm: TfmViewerMain; //our form
  end;

function ShowImageForm(ListerWin: HWND; FileToLoad: string; Fit: Boolean): HWND;
procedure HideImageForm(PluginWin: HWND);

implementation

function IsLister(ParentWindow: HWND): Boolean;
begin
  Result := GetParent(ParentWindow) = 0;
end;

//----------------------------------------------------------------------------------
{initialization and finalization}
function HookDestroy(PluginWin: HWND; Msg, wParam, lParam: LongInt): LongInt; stdcall;
var
  p: ^TPlugInfo;
begin //hook destroy our window
  p := Pointer(GetWindowLong(PluginWin, GWL_USERDATA));
  if Msg <> WM_DESTROY then
    Result := CallWindowProc(p^.PlugWinProc, PluginWin, Msg, wParam, lParam)
  else begin //plugin close
    HideImageForm(PluginWin);
    Result := 0;
  end;
end;

procedure HideImageForm(PluginWin: HWND);
var
  p: ^TPlugInfo;
begin //finalization
  p := Pointer(GetWindowLong(PluginWin, GWL_USERDATA));
  if p = nil then Exit;
  with p^.PlugForm do begin
    try
      Application.RemoveComponent(p^.PlugForm);
      Application.Handle := 0;
      //restore callback function
      SetWindowLong(Handle, GWL_WNDPROC, Integer(p^.PlugWinProc));
      Free;
    except
    end;
  end;
  Dispose(p);
end;

function ShowImageForm(ListerWin: HWND; FileToLoad: string; Fit: Boolean): HWND;
var
  p: ^TPlugInfo;
  HandRslt: THandle;
  frmMainView: TfmViewerMain;
begin //initialization
  try
    frmMainView := TfmViewerMain.CreateParented(ListerWin, FileToLoad, HandRslt, Fit);
    if HandRslt > 0 then
    begin
      frmMainView.Show;
      //synchronize our form and Lister
      Application.Handle := ListerWin;
      Application.InsertComponent(frmMainView);
      //substitution callback function
      New(p);
      SetWindowLong(frmMainView.Handle, GWL_USERDATA, Integer(p));
      p^.PlugForm := frmMainView;
      p^.PlugWinProc := Pointer(SetWindowLong(frmMainView.Handle, GWL_WNDPROC, Integer(@HookDestroy)));
      //set focus to our window
      if IsLister(frmMainView.ParentWindow) then begin
        PostMessage(frmMainView.Handle, WM_SETFOCUS, 0, 0);
      end;
      Result := frmMainView.Handle;
    end else begin
      Result := 0;
      FreeAndNil(frmMainView);
    end;
  except
    Result := 0;
  end;
end;

constructor TfmViewerMain.CreateParented(ParentWnd: HWND; FileName: string;
  var HandleResult: THandle; FitImgToWindow: Boolean);
begin
  inherited CreateParented(ParentWnd);

  HandleResult := 0;

  imgMain.Align := alClient;
  ScrollBox1.Color := clWhite;

  if not OpenImage(FileName) then Exit;
  HandleResult := Handle;
 end;

procedure TfmViewerMain.FormDestroy(Sender: TObject);
begin
  Application.ShowHint := False;
end;

procedure TfmViewerMain.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Shift = [] then
  begin
    if Key = VK_ESCAPE then PostMessage(ParentWindow,WM_KEYDOWN,VK_ESCAPE,0);
    if CharInSet(Chr(Key), ['0'..'9']) then PostMessage(ParentWindow,WM_KEYDOWN,Key,0);
  end;

  if Shift = [ssAlt] then
  begin
    if Chr(Key) = 'X' then PostMessage(ParentWindow,WM_KEYDOWN,VK_ESCAPE,0);
  end;
end;

procedure TfmViewerMain.FormResize(Sender: TObject);
begin
  with imgMain do
    Stretch := (Picture.Width > imgMain.Width) or (Picture.Height > imgMain.Height);
end;

function TfmViewerMain.GlobalLoadNextImage(FileName: string): Boolean;
begin
  Result := OpenImage(FileName);
end;

function TfmViewerMain.OpenImage(FileName: string): Boolean;
var
  Bitmap: TBitmap;
begin
  Bitmap := GetDwgThumbnail(FileName);
  try
    Result := Assigned(Bitmap);
    if Result then
      imgMain.Picture.Assign(Bitmap);
  finally
    Bitmap.Free;
  end;
end;

end.
