unit uMisc;

interface

uses Windows, SysUtils, Classes, Graphics, dwgThumbnail;

function DestSize(const CurSize, MaxSize: TPoint): TPoint;
function GetBitmapForThumb(FileName: string; AWidth, AHeight: Integer): HBitmap;

implementation

function DestSize(const CurSize, MaxSize: TPoint): TPoint;
var
  w, h, cw, ch: Integer;
  xyaspect: Double;
begin
  w := CurSize.X;
  h := CurSize.Y;
  cw := MaxSize.X;
  ch := MaxSize.Y;
	if (w > 0) and (h > 0) then
	begin
      xyaspect := w / h;
      if w > h then
      begin
        w := cw;
        h := Trunc(cw / xyaspect);
        if h > ch then  // woops, too big
        begin
          h := ch;
          w := Trunc(ch * xyaspect);
        end;
      end
      else
      begin
        h := ch;
        w := Trunc(ch * xyaspect);
        if w > cw then  // woops, too big
        begin
          w := cw;
          h := Trunc(cw / xyaspect);
        end;
      end;
    end;

  with Result do
  begin
    X := w;
    Y := h;
  end;
end;

function GetBitmapForThumb(FileName: string; AWidth, AHeight: Integer): HBitmap;
var
  DWGBitmap, SmoothBitmap: TBitmap;
  PT: TPoint;
begin
  Result := 0;
  DWGBitmap := GetDwgThumbnail(FileName);
  if not Assigned(DWGBitmap) then Exit;

  SmoothBitmap := nil;

  try
    if (DWGBitmap.Width > AWidth) or (DWGBitmap.Height > AHeight) then
    begin
      SmoothBitmap := TBitmap.Create;
      try
        PT := DestSize(Point(DWGBitmap.Width, DWGBitmap.Height),Point(AWidth, AHeight));
        SmoothBitmap.PixelFormat := pf24bit;
        SmoothBitmap.Width := PT.X;
        SmoothBitmap.Height := PT.Y;
        SetStretchBltMode(SmoothBitmap.Canvas.Handle, STRETCH_HALFTONE);
        SetBrushOrgEx(SmoothBitmap.Canvas.Handle, 0, 0, nil);
        StretchBlt(
          SmoothBitmap.Canvas.Handle,
          0, 0, SmoothBitmap.Width, SmoothBitmap.Height,
          DWGBitmap.Canvas.Handle, 0, 0, DWGBitmap.Width, DWGBitmap.Height,
          SRCCOPY);

        DWGBitmap.Assign(SmoothBitmap);
      finally
        SmoothBitmap.Free;
      end;
    end;

    DWGBitmap.HandleType := bmDDB;
    Result := DWGBitmap.ReleaseHandle;
  finally
    DWGBitmap.Free;
  end;
end;

end.
